#ifndef _DT_FFTFX_H_
#define _DT_FFTFX_H_
/**************************************************************************************************
Copyright (c) 2005-2006 Darrell Tam
email: darrell.barrell@gmail.com

FFTFx class: interface class to all DtBlkFx effect types

There is one FFTFx class instance per effect type

History
	Date       Version    Programmer         Comments
	16/2/03    1.0        Darrell Tam		Created


This program is free software; you can redistribute it and/or modify it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more details.

***************************************************************************************************/

#include <string.h>
#include <vector>
#include <dt/cplx.h>
#include "BlkFxParam.h"

class DtBlkFx;

class FFTFx
{
protected:
	// effect name
	const char* _name;

	// whether a particular fx set param is used by this effect
	bool _params_used[BlkFxParam::NUM_FX_PARAMS];

public:
	
	// construct with effect name
	FFTFx(const char* name = "?")
	{
		_name = name;

		// default to all params being used
		for(int i = 0; i < BlkFxParam::NUM_FX_PARAMS; i++) _params_used[i] = true;

	}
	virtual ~FFTFx() {}

	// DtBlkFx calls this to perform the FFT effect
	virtual void process(DtBlkFx* b) {}

	// get the value to display from the effect value buzz parameter
	virtual int/*num chars written*/dispVal(DtBlkFx* b, char* /*out*/text, float/*0..1*/val, int fx_set) 
		{ return sprintf(text, "not used"); }

	// get the effect name
	virtual const char* name() { return _name; }

	// return whether a particular FX_SET buzz parameter is used (0..NUM_FX_PARAMS-1)
	virtual bool paramUsed(int param_idx)
		{ return _params_used[param_idx]; }

	// return true if amp is used as a mix between instead of -inf to 0 dB but normal after that
	// default is amp is dB all the time
	virtual bool ampMixMode()
		{ return false; }
};

FFTFx* GetFFTFx(int idx);

// number of effects
extern const int g_num_fft_fx;

#endif
