#ifndef _BLK_FX_FX_CTRL_H_
#define _BLK_FX_FX_CTRL_H_
/**************************************************************************************************
Copyright (c) 2005-2006 Darrell Tam
email: darrell.barrell@gmail.com

Please see FxCtrl.cpp for more information

History
	Date       Version    Programmer         Comments
	01/12/05    1.0       Darrell Tam		 Created


This program is free software; you can redistribute it and/or modify it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more details.

***************************************************************************************************/

#include <memory>
#include <dt/VstGuiSupport.h>
#include <dt/VstGuiOwnerCtrls.h>

class Gui;
class DtBlkFx;

class FxCtrl : public CViewContainer, public CControlListener
// frequency control is a CViewContainer
{
public:

	FxCtrl::FxCtrl();
	FxCtrl::~FxCtrl();

	// must call before use, gui must have spectrograms init'd
	bool init(Gui* gui, CRect rect);

	// set a parameter
	void setParameter(long index, float v);

public: // override virtual methods from CView
	virtual void drawRect (CDrawContext *context, CRect& update_rect);
	virtual void draw (CDrawContext *context);

	virtual void update(CDrawContext *context);
	virtual bool isDirty() { return true; }	// cause "update" to be called every time
	virtual void mouse (CDrawContext *context, CPoint &where);
	virtual long onKeyDown(VstKeyCode& key);
	virtual long onKeyUp (VstKeyCode& key);
	virtual bool onWheel(CDrawContext *context, const CPoint &where, float distance);
	virtual void takeFocus(CDrawContext *context);
	virtual void looseFocus(CDrawContext *context);
	virtual bool removed(CView* parent);
	
public: //override virtual methods from CControlListener
	virtual void valueChanged (CDrawContext *context, CControl *control);

public: // internal stuff
	bool _isDirty();

	DtBlkFx* blkFx();

	// send the param value to the effect
	void sendParam(int param, float v);

	// what effect set we are
	int _fx_set;

	// parameter offset for this fx set
	int _param_offset;

	// parent gui
	Ptr<Gui> _gui;

	// true if we have init'd ok
	bool _ok;

	// mouse control
	MouseUpdate _mouse;

	// effect type, relative co-ordinates
	VstGuiRef<COptionMenu> _fxtype_ctrl;
	bool _fxtype_focus;

	// freq control (2 Sliders), relative coordinates
	OwnerHSlider _freq[2];
	
	// effect value control, relative co-ordinates
	OwnerHSlider _fxval;
	
	// amplitude control knob
	OwnerInvisiHSlider _amp;

	// which frequency slider should be sent to the spectrogram vline
	// -1=none, 0=_freq[0], 1=_freq[1]
	int _freq_sgram_vline;
};

#endif

