#ifndef _SINCOS_TABLE_
#define _SINCOS_TABLE_
/*
sincos table

History


This is completely free software
*/
#include <complex>
#include "cplx.h"

//******************************************************************************************
template <class T, int BITS_, class CONV=float> class SinCosTable
//
// sin cos table template (length is 1<<BITS)
// create table using "std::polar"
// overflows of 1 or -1 are ok
//
{
public:
	enum {
		BITS = BITS_,	// number of bits to index table
		LEN = 1<<BITS,	// number of elements in table
		IMASK = LEN-1	// index mask
	};

	SinCosTable() { m_table = table_(); }

	// safe for any index
	template <class T> cplxf operator [] (T i) { return m_table[i&IMASK]; }

	// direct access, no index mask
	cplxf operator () (int i) { return m_table[i]; }

protected:
	T* m_table; // cached table pointer

	static T* table_()
	{
		static bool init_table = true;
		static T ext_table[LEN+2];
		T* table = ext_table+1;
		if(init_table) {
			init_table = false;
			for(long i = 0; i < LEN; i++)
				table[i] = std::polar((CONV)1.0, (CONV)(i*2.0*3.14159265358979323846/LEN));

			// allow overflows of 1 or -1
			table[LEN] = table[0];
			table[-1] = table[LEN-1];
		}
		return table;
	}
};

#endif
