#ifndef _DT_CPLX_
#define _DT_CPLX_
/**************************************************************************************************
 Complex numbers class

 I wrote this class because std::complex didn't optimize very well in vc6 (when I checked the
 asm code it had rubbish that wouldn't optimize away)

 History
 15/3/03	Darrell Tam		created

 TODO: lots of things - I only implemented what I needed

 This software is completely free
**************************************************************************************************/
#include <complex>

// adjacent complex
struct cplxf
{
	float r, i; // real, imaginary

	cplxf(void) {}
	cplxf(float _r, float _i) { r = _r; i = _i; }
	cplxf operator () (float _r, float _i) { r = _r; i = _i; return *this; }

	cplxf(std::complex<float> a) { r = a.real(); i = a.imag(); }
	cplxf(float _r) { r = _r; i = 0.0f; }

	float real() const { return r; }
	float imag() const { return i; }

	float& real(void) { return r; }
	float& imag(void) { return i; }

	// auto-cast to std::complex
	operator std::complex<float> () { return std::complex<float>(r, i); }


};

inline cplxf operator*(const cplxf& a, const cplxf& b) { return cplxf(a.real()*b.real()-a.imag()*b.imag(), a.real()*b.imag()+a.imag()*b.real()); } 
inline cplxf operator*(float a, const cplxf& b) { return cplxf(a*b.real(), a*b.imag()); } 
inline cplxf operator*(const cplxf& a, float b) { return b*a; }
inline cplxf operator+(cplxf a, cplxf b) { return cplxf(a.real()+b.real(), a.imag()+b.imag()); } 
inline cplxf operator-(cplxf a, cplxf b) { return cplxf(a.real()-b.real(), a.imag()-b.imag()); } 
inline cplxf operator+(float a, cplxf b) { return cplxf(a+b.real(), b.imag()); } 
inline cplxf operator+(cplxf a, float b) { return b+a; }
inline cplxf operator-(float a, cplxf b) { return cplxf(a-b.real(), a-b.imag()); } 
inline cplxf operator-(cplxf a, float b) { return cplxf(a.real()-b, a.imag()); } 
inline cplxf operator-(cplxf a) { return cplxf(-a.real(), -a.imag()); } 
inline float norm(cplxf a) { return a.real()*a.real()+a.imag()*a.imag(); } 

struct pcplxf;

// dereferrenced non-adjacent complex pointer
struct dpcplxf
{
	float *r, *i;

	dpcplxf(const pcplxf& src);
	
	float real() const { return *r; }
	float imag() const { return *i; }

	float& real() { return *r; }
	float& imag() { return *i; }

	dpcplxf& operator= (const cplxf& src) { *r = src.real(); *i = src.imag(); return *this; }
	dpcplxf& operator= (const dpcplxf& src) { *r = src.real(); *i = src.imag(); return *this; }

	operator std::complex<float> () { return std::complex<float>(*r, *i); }

	// auto cast to complex value
	operator cplxf const () { return cplxf(*r, *i); }

	// address of
	//pcplxf operator & ();

protected:
	// don't want to accidentally do this
	dpcplxf(const dpcplxf& src) {}
};


// non-adjacent real & complex pointer (e.g. output from fftw)
struct pcplxf
{
	float *r, *i;

	pcplxf() {}
	pcplxf(float *_r, float *_i) { r = _r; i = _i; }
	pcplxf(float *x, long len, long offs=0) { r = x+offs; i = x+len-offs; }
	pcplxf(pcplxf src, long offs) { r = src.r+offs; i = src.i-offs; }

	pcplxf& operator() (float *x, long len, long offs) { r=x+offs; i=x+len-offs; return *this; }

	// postfix
	pcplxf operator++ (int) { pcplxf x(r, i); r++; i--; return x; }	
	pcplxf operator-- (int) { pcplxf x(r, i); r--; i++; return x; }	

	// prefix
	pcplxf& operator++ (void) { ++r; --i; return *this; }
	pcplxf& operator-- (void) { --r; ++i; return *this; }

	//
	pcplxf& operator+= (int n) { r+=n; i-=n; return *this; }
	pcplxf& operator-= (int n) { r-=n; i+=n; return *this; }

	// array
	cplxf operator [] (int a) const { return cplxf(r[a], i[-a]); }
	dpcplxf operator [] (int a) { return pcplxf(r+a, i-a); }

	// dereference
	cplxf operator * () const { return cplxf(*r, *i); }
	dpcplxf operator * () { return *this; }
};
inline pcplxf operator+(pcplxf a, long b) { return pcplxf(a.r+b, a.i-b); }
inline pcplxf operator+(long b, pcplxf a) { return a+b; }
inline pcplxf operator-(pcplxf a, long b) { return pcplxf(a.r-b, a.i+b); }

inline dpcplxf::dpcplxf(const pcplxf& src) { r = src.r; i = src.i; }
//inline pcplxf dpcplxf::operator & () { return pcplxf(r, i); }

const cplxf If(0.0f, 1.0f);

// re*re+im*im
inline float norm(float re, float im) { return norm(cplxf(re, im)); }

//------------------------------------------------------------------------------------------
struct pcplxf_range : public pcplxf
{
	float *e; // end, real

	pcplxf_range() {}

	pcplxf_range(float*x, long fft_len, long start=0, long end=0) :
		pcplxf(x, fft_len, start), e(x+end) {}

	pcplxf_range(pcplxf s, long start_offs, long end_offs)
	{
		r = s.r + start_offs;
		i = s.i - start_offs;
		e = s.r + end_offs;
	}

	// empty range
	pcplxf_range(pcplxf s)
	{
		r = s.r;
		i = s.i;
		e = s.r;
	}
	pcplxf& start() { return *this; }
	float* &end() { return e; }

	// empty once real part matches end-real
	bool empty() const { return r == e; }

};


#endif
